/*
Ultrasonic V2.0
Ultrasonic.cpp

by Cordier Yves Juin 2016 Version du 09/06/2016

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

Cette librairie permet l'utilmisation simple d'u telemete à ultra-son  Hc SR04
Fonctionne aussi avec le Hc SR05

2021/03 V2.0 Elimination d'un bug (faute de frappe)

2020/03 V2.0 Ajout de commentaires explicatifs

2016/06 V2.0 modifiée par Yves cordier 
			ajout de la prise en compte de la température 
			float RangingTemp (int unite, float T)
			ajout de la modification de l'étendue de la mesure
			void SetRange(float R)
			
2015	 V1.0 adaptation par Yves Cordier de 
			Library for HC-SR04 Ultrasonic Ranging Module.library
			Created by ITead studio. Apr 20, 2010.
			iteadstudio.com
			updated by noonv. Feb, 2011
			http://robocraft.ru
		

*/

/******************************************************************************
* Includes
******************************************************************************/
#include "Ultrasonic.h" // Attention aux majuscules/minuscules dans le nom du fichier
#include <math.h>


// ******************
// L'objet Ultrasonic
// ******************

Ultrasonic::Ultrasonic(int TP, int EP) // Le constructeur
{
	pinMode(TP, OUTPUT);
	pinMode(EP, INPUT);
	Trig_pin=TP;
	Echo_pin=EP;
	m_Last_call=micros()-Ultrasonic_delay_between_two_salves;
	m_error=Ultrasonic_no_error;
	Ultrasonic_max_range=4.0; // 4m par defaut
}

int Ultrasonic::Error() // lecture de l'erreur
{return m_error;}

long Ultrasonic::Timing() // renvoi le temps mis par le signal à faire un aller-retour
{
	digitalWrite(Trig_pin, LOW); // systeme au repos
	delayMicroseconds(2);
	digitalWrite(Trig_pin, HIGH); // armer le systeme
	delayMicroseconds(10);
	digitalWrite(Trig_pin, LOW); // lancer l'emission de la salve
	return pulseIn(Echo_pin, HIGH); // retour d'information par une impulsion caliobrée 
}

float Ultrasonic::Ranging(int unites) // résultat à 20°c par defaut
{
	return RangingTemp(unites,20);
} 

// effectue la mesure et renvoi le résultat en corrigeant en fonction de la température en degré celcius

float Ultrasonic::RangingTemp(int unites, float Temp=20)
{
	float result=-1;
	
	unsigned long T=micros();
	if (delay_between(T,m_Last_call)>Ultrasonic_delay_between_two_salves) // Une commande toute les 60 ms au maxi
	{
		long delai=Timing(); // prise de la mesure de temps
		// float Vitesse=20.063*sqrt(Temp+273);	// Calcul de la vitesse du son dans l'air en m/s
		float Vitesse=331.5+0.6071*Temp; // linéarisation au voisinage de 0 degres celcius
		if (delai<Ultrasonic_max_range*2E6/Vitesse) // on est dans les limites 4m maxi 
		{
			m_error=Ultrasonic_no_error;
			if (unites) // si !=0 en cm 
			{
				result= float(delai) *Vitesse*1E-4 ;// d=Vitesse*t/2 avec Vitesse=20.05 Sqrt(T) avec T en K
			}
			else // autrement en pouces
			{
				result= float(delai) *Vitesse*3.937E-5;// 1 pouce = 2.54 cm
			} 
		}
		else
		{
			m_error=Ultrasonic_out_of_limit;	
		}
		m_Last_call=T; // Date du dernier appel
	}
	else 
	{
		m_error=Ultrasonic_out_of_delay;
	}
	return result;
}


unsigned long Ultrasonic::delay_between(unsigned long T2, unsigned long T1) // Calcul de la durée entre seux instants en microsecondes
{
	if (T2>T1)
	{
		return(T2-T1);
	}
	else
	{
		return ( (4294967295ul-T1)+T2);
	}
}

unsigned long Ultrasonic::Last_call() // date du dernier appel
{
	return m_Last_call;
}

void Ultrasonic::SetRange(float R)
{
	Ultrasonic_max_range=R;
}

