/*
Ultrasonic V2.0
Ultrasonic.h

by Cordier Yves Juin 2016 Version du 09/06/2016

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

Cette librairie permet l'utilmisation simple d'u telemete à ultra-son  Hc SR04
Fonctionne aussi avec le Hc SR05

2021/03 V2.0 Elimination d'un bug (faute de frappe)

2016/06 V2.0 modifiée par Yves cordier 
			ajout de la prise en compte de la température 
			float RangingTemp (int unite, float T)
			ajout de la modification de l'étendue de la mesure
			void SetRange(float R)
			
2015	V1.0 adaptation par Yves Cordier de
			Library for HC-SR04 Ultrasonic Ranging Module.library
			Created by ITead studio. Apr 20, 2010.
			iteadstudio.com
			updated by noonv. Feb, 2011
			http://robocraft.ru

*/
/* 
Les fonctions de l'objet Ultrasonic :

Ultrasonic(int TP, int EP) 					: Le createur TP : pin du trigger (emeteur) EP : pin de reception (recepteur)
long Timing()								: effectue la mesure et renvoi la longueur impulsion en micro-secondes
float Ranging(int unites)					: effectue la mesure et renvoi le resultat de la mesure dans l'unité choisie 
float RangingTemp(int unites, float temp)	: effectue la mesure et renvoi le résultat en corrigeant en fonction de la température en degré celcius
int Error()									: renvoi le code d'erreur de la dernière mesure
unsigned long Last_call() 					: renvoi en milisecondes l'instant de la dernière mesure
void SetRange(float R)						: détermine la distance maximum en m (4 par defaut)

Les unités connues sont :
	Ultrasonic_unite_CM  ou CM  = 1 des centimetres
    Ultrasonic_unite_INC ou INC  = 0 des pouces !
*/
#ifndef Ultrasonic_h
#define Ultrasonic_h

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

// Pour la sortie l'unité
#define Ultrasonic_unite_CM 1
#define CM 1
#define Ultrasonic_unite_INC 0
#define INC 0

// Les erreurs codifiées
#define Ultrasonic_out_of_limit  1
#define Ultrasonic_out_of_delay 2
#define Ultrasonic_no_error 0
#define Ultrasonic_delay_between_two_salves 60000 // comme son nom l'indique
// La capacite maxi du capteur

class Ultrasonic
{
public:

	Ultrasonic(int TP, int EP); // Le createur TP : Trigger Pin EP : Echo Pin
	long Timing(); // effectue la mesure et renvoi la longueur impulsion en micro-secondes
	float Ranging(int unites); // effectue la mesure et renvoi le resultat de la mesure dans l'unité choisie
	float RangingTemp(int unites, float temp); // effectue la mesure et renvoi le résultat en corrigeant en fonction de la température en degré celcius
	int Error(); // renvoi le code d'erreur de la dernière mesure
	unsigned long Last_call(); // renvoi en milisecondes l'instant de la dernière mesure
	void SetRange(float R);
	
private:
	int Trig_pin; // Les broches trig et echo
	int Echo_pin;
	unsigned long m_Last_call; // date du dernier appel
	int m_error; // code de l'erreur 
	float Ultrasonic_max_range=4.0 ; // La distance maximum mesurée par le capteur
	unsigned long delay_between(unsigned long T2, unsigned long T1); // Calcul un delai en micro_secondes

};

#endif //#ifndef Ultrasonic_h
